@@ -4,40 +4,32 @@
  *
  * Copyright (C) 2004 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 #ifndef __COMMAND_LINE_H__
 #define __COMMAND_LINE_H__
 
-#include <NvCtrlAttributes.h>
+#include "common-utils.h"
+
+/*
+ * Forward declaration to break circular dependancy with query-assign.h
+ */
+struct _CtrlHandlesArray;
 
 #define DEFAULT_RC_FILE "~/.nvidia-settings-rc"
 #define CONFIG_FILE_OPTION 1
 
-
-#define VERBOSITY_NONE     0 /* nothing */
-#define VERBOSITY_ERROR    1 /* errors only */
-#define VERBOSITY_WARNING  2 /* errors and warnings */
-#define VERBOSITY_ALL      3 /* errors, warnings and other info */
-
-#define VERBOSITY_DEFAULT  VERBOSITY_ERROR
-
 /*
  * Options structure -- stores the parameters specified on the
  * commandline.
@@ -102,6 +94,7 @@ typedef struct {
 } Options;
 
 
-Options *parse_command_line(int argc, char *argv[], char *dpy);
+Options *parse_command_line(int argc, char *argv[], char *dpy, 
+                            struct _CtrlHandlesArray *handles_array);
 
 #endif /* __COMMAND_LINE_H__ */
