@@ -65,7 +65,8 @@ static ParsedAttributeWrapper *parse_config_file(char *buf,
 
 static int process_config_file_attributes(const char *file,
                                           ParsedAttributeWrapper *w,
-                                          const char *display_name);
+                                          const char *display_name,
+                                          CtrlHandlesArray *handles_array);
 
 static void save_gui_parsed_attributes(ParsedAttributeWrapper *w,
                                        ParsedAttribute *p);
@@ -101,7 +102,8 @@ extern int __verbosity_level_changed;
  */
 
 int nv_read_config_file(const char *file, const char *display_name,
-                        ParsedAttribute *p, ConfigProperties *conf)
+                        ParsedAttribute *p, ConfigProperties *conf,
+                        CtrlHandlesArray *handles_array)
 {
     int fd, ret, length;
     struct stat stat_buf;
@@ -180,7 +182,8 @@ int nv_read_config_file(const char *file, const char *display_name,
 
     /* process the parsed attributes */
 
-    ret = process_config_file_attributes(file, w, display_name);
+    ret = process_config_file_attributes(file, w, display_name, 
+                                         handles_array);
 
     /*
      * add any relevant parsed attributes back to the list to be
@@ -211,10 +214,9 @@ int nv_read_config_file(const char *file, const char *display_name,
 int nv_write_config_file(const char *filename, CtrlHandles *h,
                          ParsedAttribute *p, ConfigProperties *conf)
 {
-    int screen, ret, entry, bit, val;
+    int screen, ret, entry, bit, val, display, randr_gamma_available;
     FILE *stream;
     time_t now;
-    AttributeTableEntry *a;
     ReturnStatus status;
     NVCTRLAttributeValidValuesRec valid;
     uint32 mask;
@@ -272,8 +274,8 @@ int nv_write_config_file(const char *filename, CtrlHandles *h,
     fprintf(stream, "\n");
 
     /*
-     * Note: we only save writable attributes addressable by X screen
-     * (i.e., we don't look at other target types, yet).
+     * Note: we only save writable attributes addressable by X screen here
+     * followed by attributes for display target types.
      */
     
     for (screen = 0; screen < h->targets[X_SCREEN_TARGET].n; screen++) {
@@ -301,9 +303,8 @@ int nv_write_config_file(const char *filename, CtrlHandles *h,
         /* loop over all the entries in the table */
 
         for (entry = 0; attributeTable[entry].name; entry++) {
+            const AttributeTableEntry *a = &attributeTable[entry];
 
-            a = &attributeTable[entry];
-            
             /* 
              * skip all attributes that are not supposed to be written
              * to the config file
@@ -318,6 +319,14 @@ int nv_write_config_file(const char *filename, CtrlHandles *h,
             
             if (a->flags & NV_PARSER_TYPE_COLOR_ATTRIBUTE) {
                 float c[3], b[3], g[3];
+
+                /*
+                 * if we are using RandR gamma, skip saving the color info
+                 */
+ 
+                status = NvCtrlGetAttribute(t->h, NV_CTRL_ATTR_RANDR_GAMMA_AVAILABLE, &val);
+                if (status == NvCtrlSuccess && val) continue;
+
                 status = NvCtrlGetColorAttributes(t->h, c, b, g);
                 if (status != NvCtrlSuccess) continue;
                 fprintf(stream, "%s%c%s=%f\n",
@@ -379,6 +388,68 @@ int nv_write_config_file(const char *filename, CtrlHandles *h,
         } /* entry */
         
     } /* screen */
+
+    /*
+     * Write attributes addresssable to display targets
+     */
+
+    for (display = 0; display < h->targets[DISPLAY_TARGET].n; display++) {
+
+        t = &h->targets[DISPLAY_TARGET].t[display];
+
+        /* skip it if we don't have a handle for this display */
+
+        if (!t->h) continue;
+
+        /* 
+         * check to see if we have RANDR gamma available. We may
+         * skip writing attributes if it is missing. 
+         */
+
+        status = NvCtrlGetAttribute(t->h, 
+                                    NV_CTRL_ATTR_RANDR_GAMMA_AVAILABLE,
+                                    &randr_gamma_available);
+        if (status != NvCtrlSuccess) {
+            randr_gamma_available = 0;
+        }
+
+        /*
+         * use the full display name as the prefix for display targets
+         * until we can parse other display target names
+         */
+
+        prefix = t->name;
+
+        /* loop over all the entries in the table */
+
+        for (entry = 0; attributeTable[entry].name; entry++) {
+            const AttributeTableEntry *a = &attributeTable[entry];
+
+            /* 
+             * skip all attributes that are not supposed to be written
+             * to the config file
+             */
+
+            if (a->flags & NV_PARSER_TYPE_NO_CONFIG_WRITE) continue;
+
+            /* 
+             * for the display target we only write color attributes for now
+             */
+
+            if (a->flags & NV_PARSER_TYPE_COLOR_ATTRIBUTE) {
+                float c[3], b[3], g[3];
+
+                if (!randr_gamma_available) continue;
+
+                status = NvCtrlGetColorAttributes(t->h, c, b, g);
+                if (status != NvCtrlSuccess) continue;
+
+                fprintf(stream, "%s%c%s=%f\n",
+                        prefix, DISPLAY_NAME_SEPARATOR, a->name,
+                        get_color_value(a->attr, c, b, g));
+            } 
+        }
+    }
     
     /*
      * loop the ParsedAttribute list, writing the attributes to file.
@@ -410,40 +481,38 @@ int nv_write_config_file(const char *filename, CtrlHandles *h,
          * target type other than an X screen, include a target
          * specification in what we write to the .rc file.
          */
-        
+
         target_str[0] = '\0';
-        
+
         if ((p->flags & NV_PARSER_HAS_TARGET) &&
             (p->target_type != NV_CTRL_TARGET_TYPE_X_SCREEN)) {
-            
-            int j;
-            
+
+            const TargetTypeEntry *targetTypeEntry;
+
             /* Find the target name of the target type */
-            for (j = 0; targetTypeTable[j].name; j++) {
-                if (targetTypeTable[j].nvctrl == p->target_type) {
-                    snprintf(target_str, 64, "[%s:%d]",
-                             targetTypeTable[j].parsed_name, p->target_id);
-                    break;
-                }
+            targetTypeEntry = nv_get_target_type_entry_by_nvctrl(p->target_type);
+            if (targetTypeEntry) {
+                snprintf(target_str, 64, "[%s:%d]",
+                         targetTypeEntry->parsed_name, p->target_id);
             }
         }
-        
+
         if (p->display_device_mask) {
-            
+
             tmp_d_str = display_device_mask_to_display_device_name
                 (p->display_device_mask);
-            
+
             fprintf(stream, "%s%s%c%s[%s]=%d\n", p->display, target_str,
-                    DISPLAY_NAME_SEPARATOR, tmp, tmp_d_str, p->val);
-            
+                    DISPLAY_NAME_SEPARATOR, tmp, tmp_d_str, p->val.i);
+
             free(tmp_d_str);
-            
+
         } else {
-                
+
             fprintf(stream, "%s%s%c%s=%d\n", p->display, target_str,
-                    DISPLAY_NAME_SEPARATOR, tmp, p->val);
+                    DISPLAY_NAME_SEPARATOR, tmp, p->val.i);
         }
-        
+
         p = p->next;
     }
 
@@ -497,8 +566,7 @@ static ParsedAttributeWrapper *parse_config_file(char *buf, const char *file,
         
         while (((c - buf) < length) &&
                (*c != '\n') &&
-               (*c != '\0') &&
-               (*c != EOF)) {
+               (*c != '\0')) {
             if (comment) { c++; continue; }
             if (*c == '#') { comment = c; continue; }
             if (!isspace(*c)) has_data = NV_TRUE;
@@ -516,7 +584,7 @@ static ParsedAttributeWrapper *parse_config_file(char *buf, const char *file,
                 if (tmp) {
                     free(tmp);
                 }
-                tmp = malloc(sizeof(char) * current_tmp_len);
+                tmp = nvalloc(sizeof(char) * current_tmp_len);
             }
 
             strncpy (tmp, cur, len);
@@ -526,7 +594,7 @@ static ParsedAttributeWrapper *parse_config_file(char *buf, const char *file,
 
             if (!parse_config_property(file, tmp, conf)) {
                 
-                w = realloc(w, sizeof(ParsedAttributeWrapper) * (n+1));
+                w = nvrealloc(w, sizeof(ParsedAttributeWrapper) * (n+1));
             
                 ret = nv_parse_attribute_string(tmp,
                                                 NV_PARSER_ASSIGNMENT,
@@ -543,15 +611,15 @@ static ParsedAttributeWrapper *parse_config_file(char *buf, const char *file,
             }
         }
 
-        if (((c - buf) >= length) || (*c == '\0') || (*c == EOF)) cur = NULL;
+        if (((c - buf) >= length) || (*c == '\0')) cur = NULL;
         else cur = c + 1;
-        
+
         line++;
     }
     free(tmp);
     /* mark the end of the array */
 
-    w = realloc(w, sizeof(ParsedAttributeWrapper) * (n+1));
+    w = nvrealloc(w, sizeof(ParsedAttributeWrapper) * (n+1));
     w[n].line = -1;
     
     return w;
@@ -574,10 +642,10 @@ static ParsedAttributeWrapper *parse_config_file(char *buf, const char *file,
 
 static int process_config_file_attributes(const char *file,
                                           ParsedAttributeWrapper *w,
-                                          const char *display_name)
+                                          const char *display_name,
+                                          CtrlHandlesArray *handles_array)
 {
-    int i, j, ret, found, n = 0;
-    CtrlHandles **h = NULL;
+    int i;
     
     int old_verbosity = __verbosity;
 
@@ -600,41 +668,17 @@ static int process_config_file_attributes(const char *file,
     /* build the list of CtrlHandles */
     
     for (i = 0; w[i].line != -1; i++) {
-        found = NV_FALSE;
-        for (j = 0; j < n; j++) {
-            if (nv_strcasecmp(h[j]->display, w[i].a.display)) {
-                w[i].h = h[j];
-                found = NV_TRUE;
-                break;
-            }
-        }
-
-        /*
-         * no handle found for this display, need to create a new
-         * handle.
-         *
-         * XXX we should really just build a list of what ctrl_handles
-         * we need, and what attributes on which ctrl_handles, so that
-         * we don't have to pass NV_CTRL_ATTRIBUTES_ALL_SUBSYSTEMS to
-         * NvCtrlAttributeInit (done in nv_alloc_ctrl_handles())
-         * unless we really need it.
-         */
-
-        if (!found) {
-            h = realloc(h, sizeof(CtrlHandles *) * (n + 1));
-            h[n] = nv_alloc_ctrl_handles(w[i].a.display);
-            w[i].h = h[n];
-            n++;
-        }
+        w[i].h = nv_alloc_ctrl_handles_and_add_to_array(w[i].a.display, 
+                                                        handles_array);
     }
     
     /* now process each attribute, passing in the correct CtrlHandles */
 
     for (i = 0; w[i].line != -1; i++) {
 
-        ret = nv_process_parsed_attribute(&w[i].a, w[i].h, NV_TRUE, NV_FALSE,
-                                          "on line %d of configuration file "
-                                          "'%s'", w[i].line, file);
+        nv_process_parsed_attribute(&w[i].a, w[i].h, NV_TRUE, NV_FALSE,
+                                    "on line %d of configuration file "
+                                    "'%s'", w[i].line, file);
         /*
          * We do not fail if processing the attribute failed.  If the
          * GPU or the X config changed (for example stereo is
@@ -650,15 +694,6 @@ static int process_config_file_attributes(const char *file,
         __verbosity = old_verbosity;
     }
 
-    /* free all the CtrlHandles we allocated */
-
-    for (i = 0; i < n; i++) {
-        nv_free_ctrl_handles(h[i]);
-    }
-    
-    if (h) free(h);
-
-
     return NV_TRUE;
     
 } /* process_config_file_attributes() */
@@ -723,6 +758,8 @@ ConfigPropertiesTableEntry configPropertyTable[] = {
     { "IncludeDisplayNameInConfigFile",
       CONFIG_PROPERTIES_INCLUDE_DISPLAY_NAME_IN_CONFIG_FILE },
     { "ShowQuitDialog", CONFIG_PROPERTIES_SHOW_QUIT_DIALOG },
+    { "UpdateRulesOnProfileNameChange",
+      CONFIG_PROPERTIES_UPDATE_RULES_ON_PROFILE_NAME_CHANGE },
     { NULL, 0 }
 };
 
@@ -770,23 +807,9 @@ static int parse_config_property(const char *file, const char *line, ConfigPrope
         if (!token)
             goto done;
 
-        c = malloc(sizeof(TimerConfigProperty));
-        if (!c) {
-            nv_warning_msg("Error parsing configuration file '%s': could "
-                           "not allocate memory for timer '%s'.",
-                           file, timer);
-            ret = NV_TRUE;
-            goto done;
-        }
+        c = nvalloc(sizeof(TimerConfigProperty));
 
         c->description = replace_characters(token, '_', ' ');
-        if (!c->description) {
-            nv_warning_msg("Error parsing configuration file '%s': could "
-                           "not allocate memory for timer '%s'.",
-                           file, timer);
-            ret = NV_TRUE;
-            goto done;
-        }
 
         token = strtok(NULL, ",");
         if (!token)
@@ -872,8 +895,6 @@ static void write_config_properties(FILE *stream, ConfigProperties *conf, char *
 
     for (c = conf->timers; (c != NULL); c = c->next) {
         description = replace_characters(c->description, ' ', '_');
-        if (!description)
-            continue;
         fprintf(stream, "Timer = %s,%s,%u\n",
                 description, c->user_enabled ? "Yes" : "No",
                 c->interval);
@@ -896,7 +917,8 @@ void init_config_properties(ConfigProperties *conf)
         (CONFIG_PROPERTIES_TOOLTIPS |
          CONFIG_PROPERTIES_DISPLAY_STATUS_BAR |
          CONFIG_PROPERTIES_SLIDER_TEXT_ENTRIES |
-         CONFIG_PROPERTIES_SHOW_QUIT_DIALOG);
+         CONFIG_PROPERTIES_SHOW_QUIT_DIALOG |
+         CONFIG_PROPERTIES_UPDATE_RULES_ON_PROFILE_NAME_CHANGE);
 
     conf->locale = strdup(setlocale(LC_NUMERIC, NULL));
 
