@@ -4,22 +4,17 @@
  *
  * Copyright (C) 2004 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 #include <gtk/gtk.h>
@@ -28,10 +23,10 @@
 #include <stdlib.h>
 #include <string.h>
 #include <X11/Xlib.h>
-#include <X11/extensions/Xrandr.h>
 
 #include "msg.h"
 #include "parse.h"
+#include "common-utils.h"
 
 #include "ctkbanner.h"
 
@@ -39,6 +34,7 @@
 #include "ctkdisplayconfig-utils.h"
 #include "ctkhelp.h"
 #include "ctkutils.h"
+#include "ctkdropdownmenu.h"
 
 
 /* Static function declarations */
@@ -76,24 +72,53 @@ typedef struct DpyLocRec { // Display Location
 
 
 
-/** 
+/**
  * The gridConfigs array enumerates the display grid configurations
  * that are presently supported.
  *
  **/
 
 static GridConfig gridConfigs[] = {
-    {2, 2, FALSE}, // rows, columns, valid
-    {2, 3, FALSE},
-    {2, 4, FALSE},
-    {3, 1, FALSE},
-    {3, 2, FALSE},
-    {1, 3, FALSE},
-    {2, 1, FALSE},
-    {1, 2, FALSE},
-    {4, 1, FALSE},
-    {1, 4, FALSE},
-    {0, 0, FALSE}
+    {4, 4,  FALSE}, // rows, columns, valid
+    {5, 3,  FALSE},
+    {3, 5,  FALSE},
+    {3, 4,  FALSE},
+    {4, 3,  FALSE},
+    {3, 3,  FALSE},
+    {2, 8,  FALSE},
+    {8, 2,  FALSE},
+    {2, 7,  FALSE},
+    {7, 2,  FALSE},
+    {2, 6,  FALSE},
+    {6, 2,  FALSE},
+    {2, 5,  FALSE},
+    {5, 2,  FALSE},
+    {1, 16, FALSE},
+    {16, 1, FALSE},
+    {1, 15, FALSE},
+    {15, 1, FALSE},
+    {1, 14, FALSE},
+    {14, 1, FALSE},
+    {1, 13, FALSE},
+    {13, 1, FALSE},
+    {1, 12, FALSE},
+    {12, 1, FALSE},
+    {1, 11, FALSE},
+    {11, 1, FALSE},
+    {1, 10, FALSE},
+    {10, 1, FALSE},
+    {1, 9,  FALSE},
+    {9, 1,  FALSE},
+    {2, 2,  FALSE},
+    {2, 3,  FALSE},
+    {2, 4,  FALSE},
+    {3, 1,  FALSE},
+    {3, 2,  FALSE},
+    {1, 3,  FALSE},
+    {2, 1,  FALSE},
+    {1, 2,  FALSE},
+    {4, 1,  FALSE},
+    {1, 4,  FALSE},
 };
 
 GType ctk_slimm_get_type()
@@ -111,6 +136,7 @@ GType ctk_slimm_get_type()
             sizeof (CtkSLIMM),
             0, /* n_preallocs */
             NULL, /* instance_init */
+            NULL  /* value_table */
         };
 
         ctk_slimm_type =
@@ -141,7 +167,7 @@ static void remove_slimm_options(XConfigPtr xconf)
 static GridConfig *get_ith_valid_grid_config(int idx)
 {
     int i, count = 0;
-    for (i = 0; gridConfigs[i].rows; i++) {
+    for (i = 0; i < ARRAY_LEN(gridConfigs); i++) {
         if (!gridConfigs[i].valid) continue;
         if (count == idx) return &gridConfigs[i];
         count++;
@@ -196,6 +222,7 @@ static XConfigPtr xconfig_generate(XConfigPtr xconfCur,
                                    void *callback_data)
 {
     CtkSLIMM *ctk_object = (CtkSLIMM *)callback_data;
+    CtkDropDownMenu *menu = CTK_DROP_DOWN_MENU(ctk_object->mnu_display_config);
 
     gint idx;
 
@@ -204,8 +231,6 @@ static XConfigPtr xconfig_generate(XConfigPtr xconfCur,
     gint x_displays,y_displays;
     gint h_overlap, v_overlap;
 
-    gint x_total, y_total;
-
     gchar *metamode_str = NULL;
     gchar *tmpstr;
 
@@ -223,7 +248,7 @@ static XConfigPtr xconfig_generate(XConfigPtr xconfCur,
     if (checkbox_state) {
         GridConfig *grid_config;
         /* SLI MM needs to be enabled */
-        idx = gtk_option_menu_get_history(GTK_OPTION_MENU(ctk_object->mnu_display_config));
+        idx = ctk_drop_down_menu_get_current_value(menu);
 
         /* Get grid configuration values from index */
 
@@ -238,12 +263,6 @@ static XConfigPtr xconfig_generate(XConfigPtr xconfCur,
         h_overlap = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ctk_object->spbtn_hedge_overlap));
         v_overlap = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ctk_object->spbtn_vedge_overlap));
 
-        /* Total X Screen Size Calculation */
-        x_total = x_displays * ctk_object->cur_modeline->data.hdisplay - 
-                  (x_displays - 1) * h_overlap;
-        y_total = y_displays * ctk_object->cur_modeline->data.vdisplay - 
-                  (y_displays - 1) * v_overlap;
-
         for (yctr = 0; yctr < y_displays;yctr++) {
             for (xctr = 0; xctr < x_displays;xctr++) {
                 tmpstr = g_strdup_printf("%s +%d+%d",
@@ -283,12 +302,15 @@ static Bool compute_screen_size(CtkSLIMM *ctk_object, gint *width,
     gint x_displays,y_displays;
     gint h_overlap, v_overlap;
 
+    CtkDropDownMenu *menu;
+
 
     if (!ctk_object->cur_modeline) {
         return FALSE;
     }
 
-    config_idx = gtk_option_menu_get_history(GTK_OPTION_MENU(ctk_object->mnu_display_config));
+    menu = CTK_DROP_DOWN_MENU(ctk_object->mnu_display_config);
+    config_idx = ctk_drop_down_menu_get_current_value(menu);
 
     /* Get grid configuration values from index */
     grid_config = get_ith_valid_grid_config(config_idx);
@@ -353,7 +375,7 @@ static void save_xconfig_button_clicked(GtkWidget *widget, gpointer user_data)
              GTK_DIALOG_DESTROY_WITH_PARENT,
              GTK_MESSAGE_WARNING,
              GTK_BUTTONS_OK,
-             err_msg);
+             "%s", err_msg);
             
         gtk_dialog_run(GTK_DIALOG(dlg));
         gtk_widget_destroy(dlg);
@@ -386,10 +408,11 @@ static void display_config_changed(GtkWidget *widget, gpointer user_data)
 static void display_refresh_changed(GtkWidget *widget, gpointer user_data)
 {
     CtkSLIMM *ctk_object = CTK_SLIMM(user_data);
+    CtkDropDownMenu *menu = CTK_DROP_DOWN_MENU(widget);
     gint idx;
 
     /* Get the modeline and display to set */
-    idx = gtk_option_menu_get_history(GTK_OPTION_MENU(widget));
+    idx = ctk_drop_down_menu_get_current_value(menu);
 
     /* Select the new modeline as current modeline */
     ctk_object->cur_modeline = ctk_object->refresh_table[idx];
@@ -399,12 +422,13 @@ static void display_refresh_changed(GtkWidget *widget, gpointer user_data)
 static void display_resolution_changed(GtkWidget *widget, gpointer user_data)
 {
     CtkSLIMM *ctk_object = CTK_SLIMM(user_data);
+    CtkDropDownMenu *menu = CTK_DROP_DOWN_MENU(widget);
 
     gint idx;
     nvModeLinePtr modeline;
 
     /* Get the modeline and display to set */
-    idx = gtk_option_menu_get_history(GTK_OPTION_MENU(widget));
+    idx = ctk_drop_down_menu_get_current_value(menu);
     modeline = ctk_object->resolution_table[idx];
 
     /* Ignore selecting same resolution */
@@ -499,8 +523,7 @@ static void setup_total_size_label(CtkSLIMM *ctk_object)
 
 static void setup_display_refresh_dropdown(CtkSLIMM *ctk_object)
 {
-    GtkWidget *menu;
-    GtkWidget *menu_item;
+    CtkDropDownMenu *menu;
     nvModeLinePtr modeline;
     float cur_rate; /* Refresh Rate */
     int cur_idx = 0; /* Currently selected modeline */
@@ -522,14 +545,20 @@ static void setup_display_refresh_dropdown(CtkSLIMM *ctk_object)
         ctk_object->refresh_table_len = 0;
     }
     ctk_object->refresh_table =
-        (nvModeLinePtr *)calloc(ctk_object->num_modelines, sizeof(nvModeLinePtr));
+        calloc(ctk_object->num_modelines, sizeof(nvModeLinePtr));
     if (!ctk_object->refresh_table) {
         goto fail;
     }
 
 
     /* Generate the refresh dropdown */
-    menu = gtk_menu_new();
+    menu = CTK_DROP_DOWN_MENU(ctk_object->mnu_display_refresh);
+    
+    ctk_drop_down_menu_reset(menu);
+
+    g_signal_handlers_block_by_func
+        (G_OBJECT(ctk_object->mnu_display_refresh),
+         G_CALLBACK(display_refresh_changed), (gpointer) ctk_object);
 
     /* Generate the refresh rate dropdown from the modelines list */
     for (modeline = ctk_object->modelines; modeline; modeline = modeline->next) {
@@ -538,8 +567,6 @@ static void setup_display_refresh_dropdown(CtkSLIMM *ctk_object)
         nvModeLinePtr m;
         int count_ref; /* # modelines with similar refresh rates */
         int num_ref;   /* Modeline # in a group of similar refresh rates */
-        int is_doublescan;
-        int is_interlaced;
 
         gchar *extra = NULL;
         gchar *tmp;
@@ -551,8 +578,6 @@ static void setup_display_refresh_dropdown(CtkSLIMM *ctk_object)
         }
 
         modeline_rate = modeline->refresh_rate;
-        is_doublescan = (modeline->data.flags & V_DBLSCAN);
-        is_interlaced = (modeline->data.flags & V_INTERLACE);
 
         name = g_strdup_printf("%.0f Hz", modeline_rate);
 
@@ -637,22 +662,14 @@ static void setup_display_refresh_dropdown(CtkSLIMM *ctk_object)
 
 
         /* Add the modeline entry to the dropdown */
-        menu_item = gtk_menu_item_new_with_label(name);
+        ctk_drop_down_menu_append_item(menu, name, ctk_object->refresh_table_len);
         g_free(name);
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
-        gtk_widget_show(menu_item);
         ctk_object->refresh_table[ctk_object->refresh_table_len++] = modeline;
     }
 
     /* Setup the menu and select the current mode */
-    g_signal_handlers_block_by_func
-        (G_OBJECT(ctk_object->mnu_display_refresh),
-         G_CALLBACK(display_refresh_changed), (gpointer) ctk_object);
-
     ctk_object->cur_modeline = ctk_object->refresh_table[cur_idx];
-
-    gtk_option_menu_set_menu(GTK_OPTION_MENU(ctk_object->mnu_display_refresh), menu);
-    gtk_option_menu_set_history(GTK_OPTION_MENU(ctk_object->mnu_display_refresh), cur_idx);
+    ctk_drop_down_menu_set_current_value(menu, cur_idx);
     gtk_widget_set_sensitive(ctk_object->mnu_display_refresh, True);
 
     g_signal_handlers_unblock_by_func
@@ -680,8 +697,7 @@ static void setup_display_refresh_dropdown(CtkSLIMM *ctk_object)
 
 static void setup_display_resolution_dropdown(CtkSLIMM *ctk_object)
 {
-    GtkWidget *menu;
-    GtkWidget *menu_item;
+    CtkDropDownMenu *menu;
 
     nvModeLinePtr  modeline;
     nvModeLinePtr  cur_modeline = ctk_object->cur_modeline;
@@ -694,18 +710,20 @@ static void setup_display_resolution_dropdown(CtkSLIMM *ctk_object)
         ctk_object->resolution_table_len = 0;
     }
     ctk_object->resolution_table =
-        (nvModeLinePtr *)calloc((ctk_object->num_modelines + 1),
-                                sizeof(nvModeLinePtr));
+        calloc((ctk_object->num_modelines + 1), sizeof(nvModeLinePtr));
     if (!ctk_object->resolution_table) {
         goto fail;
     }
 
     /* Start the menu generation */
-    menu = gtk_menu_new();
+    menu = CTK_DROP_DOWN_MENU(ctk_object->mnu_display_resolution);
 
     modeline = ctk_object->modelines;
     cur_idx = 0;
 
+    g_signal_handlers_block_by_func
+        (G_OBJECT(ctk_object->mnu_display_resolution),
+         G_CALLBACK(display_resolution_changed), (gpointer) ctk_object);
     /* Generate the resolution menu */
 
     while (modeline) {
@@ -736,10 +754,9 @@ static void setup_display_resolution_dropdown(CtkSLIMM *ctk_object)
 
             name = g_strdup_printf("%dx%d", modeline->data.hdisplay,
                                    modeline->data.vdisplay);
-            menu_item = gtk_menu_item_new_with_label(name);
+            ctk_drop_down_menu_append_item(menu, name,
+                                           ctk_object->resolution_table_len);
             g_free(name);
-            gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
-            gtk_widget_show(menu_item);
             ctk_object->resolution_table[ctk_object->resolution_table_len++] =
                 modeline;
         }
@@ -747,15 +764,8 @@ static void setup_display_resolution_dropdown(CtkSLIMM *ctk_object)
     }
 
     /* Setup the menu and select the current mode */
-    g_signal_handlers_block_by_func
-        (G_OBJECT(ctk_object->mnu_display_resolution),
-         G_CALLBACK(display_resolution_changed), (gpointer) ctk_object);
 
-    gtk_option_menu_set_menu
-        (GTK_OPTION_MENU(ctk_object->mnu_display_resolution), menu);
-
-    gtk_option_menu_set_history
-        (GTK_OPTION_MENU(ctk_object->mnu_display_resolution), cur_idx);
+    ctk_drop_down_menu_set_current_value(menu, cur_idx);
 
     /* If dropdown has only one item, disable menu selection */
     if (ctk_object->resolution_table_len > 1) {
@@ -773,9 +783,6 @@ static void setup_display_resolution_dropdown(CtkSLIMM *ctk_object)
     /* Handle failures */
  fail:
 
-    gtk_option_menu_remove_menu
-        (GTK_OPTION_MENU(ctk_object->mnu_display_resolution));
-
     gtk_widget_set_sensitive(ctk_object->mnu_display_resolution, False);
 
 } /* setup_display_resolution_dropdown() */
@@ -846,20 +853,20 @@ static Bool parse_slimm_layout(CtkSLIMM *ctk_slimm,
 
     /* Make space for the display location array */
     if (!locs) {
-        for (loc_idx = 0; gridConfigs[loc_idx].rows; loc_idx++ ) {
+        for (loc_idx = 0; loc_idx < ARRAY_LEN(gridConfigs); loc_idx++ ) {
             if ( max_rows < gridConfigs[loc_idx].rows) {
                 max_rows = gridConfigs[loc_idx].rows;
             }
             if ( max_cols < gridConfigs[loc_idx].columns) {
                 max_cols = gridConfigs[loc_idx].columns;
             }
-            if (max_locs < 
+            if (max_locs <
                 (gridConfigs[loc_idx].rows * gridConfigs[loc_idx].columns)) {
                 max_locs =
                     gridConfigs[loc_idx].rows * gridConfigs[loc_idx].columns;
             }
         }
-        locs = (DpyLoc *)malloc(max_locs * sizeof(DpyLoc));
+        locs = malloc(max_locs * sizeof(DpyLoc));
         if (!locs) {
             err_msg = "Out of memory.";
             goto fail;
@@ -1016,10 +1023,10 @@ static Bool parse_slimm_layout(CtkSLIMM *ctk_slimm,
                 goto fail;
             }
         }
-        
+
         /* Make sure this is a known/supported grid config */
         found = 0;
-        for (i = 0; gridConfigs[i].rows; i++) {
+        for (i = 0; i < ARRAY_LEN(gridConfigs); i++) {
             if ((gridConfigs[i].rows == rows) &&
                 (gridConfigs[i].columns == cols)) {
                 *grid_config_id = i;
@@ -1150,8 +1157,8 @@ static Bool other_displays_have_modeline(nvLayoutPtr layout,
     nvGpuPtr gpu;
     nvDisplayPtr d;
 
-    for (gpu = layout->gpus; gpu; gpu = gpu->next) {
-        for (d = gpu->displays; d; d = d->next) {
+    for (gpu = layout->gpus; gpu; gpu = gpu->next_in_layout) {
+        for (d = gpu->displays; d; d = d->next_on_gpu) {
             if (display == d) continue;
             if (d->modelines == NULL) continue;
             if (!display_has_modeline(d, modeline)) {
@@ -1168,8 +1175,11 @@ static nvDisplayPtr find_active_display(nvLayoutPtr layout)
 {
     nvGpuPtr gpu;
     nvDisplayPtr display;
-    for (gpu = layout->gpus; gpu; gpu = gpu->next) {
-        for (display = gpu->displays; display; display = display->next) {
+
+    for (gpu = layout->gpus; gpu; gpu = gpu->next_in_layout) {
+        for (display = gpu->displays;
+             display;
+             display = display->next_on_gpu) {
             if (display->modelines) return display;
         }
     }
@@ -1241,13 +1251,16 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
     GtkWidget *table;
     GtkWidget *button;
 
-    GtkWidget *optionmenu, *menu, *menuitem, *spinbutton;
+    GtkWidget *spinbutton;
     CtkSLIMM *ctk_object;
+    CtkDropDownMenu *menu;
 
     gchar *err_str = NULL;
     gchar *tmp;
     gchar *sli_mode = NULL;
     ReturnStatus ret;
+    ReturnStatus ret1;
+    int major = 0, minor = 0;
     gint val;
 
     nvLayoutPtr layout;
@@ -1257,7 +1270,8 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
     int grid_menu_selected_id;
     int count;
 
-    Bool valid_layout = FALSE;
+    Bool trust_slimm_available = FALSE;
+    int vcs_target_count;
 
     int hoverlap = 0;
     int voverlap = 0;
@@ -1273,7 +1287,29 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
     ctk_slimm->handle = handle;
     ctk_slimm->ctk_config = ctk_config;
     ctk_object = ctk_slimm;
+
+    /*
+     * Check for NV-CONTROL protocol version.
+     * This used for not trust old X drivers which always reported
+     * it available (on NV50+).
+     */
+    ret = NvCtrlGetAttribute(handle,
+                             NV_CTRL_ATTR_NV_MAJOR_VERSION, &major);
+    ret1 = NvCtrlGetAttribute(handle,
+                              NV_CTRL_ATTR_NV_MINOR_VERSION, &minor);
+
+    if ((ret == NvCtrlSuccess) && (ret1 == NvCtrlSuccess) &&
+        ((major > 1) || ((major == 1) && (minor > 23)))) {
+        trust_slimm_available = TRUE;
+    }
     
+    vcs_target_count = ctk_config->pCtrlHandles->targets[VCS_TARGET].n;
+
+    /* return on old X drivers if target is other than VCS. */
+    if (!vcs_target_count && !trust_slimm_available) {
+      return NULL;
+    }
+
     /* Check if this screen supports SLI Mosaic Mode */
     ret = NvCtrlGetAttribute(ctk_object->handle,
                              NV_CTRL_SLI_MOSAIC_MODE_AVAILABLE, &val);
@@ -1310,15 +1346,17 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
         nvGpuPtr gpu;
         int num_displays = 0;
         int min_displays = 0;
-        GridConfig *grid;
+        int i;
         int num_valid = 0;
 
-        for (gpu = layout->gpus; gpu; gpu = gpu->next) {
+        for (gpu = layout->gpus; gpu; gpu = gpu->next_in_layout) {
             num_displays += gpu->num_displays;
         }
 
         /* Mark configs that have enough displays as valid */
-        for (grid = gridConfigs; grid->rows; grid++) {
+        for (i = 0; i < ARRAY_LEN(gridConfigs); i++) {
+            GridConfig *grid = &(gridConfigs[i]);
+
             if (!min_displays ||
                 (min_displays > (grid->rows * grid->columns))) {
                 min_displays = grid->rows * grid->columns;
@@ -1328,7 +1366,7 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
                 num_valid++;
             }
         }
-        
+
         /* Make sure we have enough dislays for the minimum config */
         if (num_valid <= 0) {
             err_str = g_strdup_printf("Not enough display devices to "
@@ -1343,11 +1381,11 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
             layout = NULL;
 
         } else {
-            valid_layout = parse_slimm_layout(ctk_slimm,
-                                              layout,
-                                              &hoverlap,
-                                              &voverlap,
-                                              &grid_config_id);
+            parse_slimm_layout(ctk_slimm,
+                               layout,
+                               &hoverlap,
+                               &voverlap,
+                               &grid_config_id);
         }
     }
     
@@ -1447,22 +1485,20 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
     hbox = gtk_hbox_new(FALSE, 0);
 
     /* Option menu for Display Grid Configuration */
-    optionmenu = gtk_option_menu_new();
-    ctk_slimm->mnu_display_config = optionmenu;
-    menu = gtk_menu_new();
+    menu = (CtkDropDownMenu *)
+        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_COMBO);
+    ctk_slimm->mnu_display_config = GTK_WIDGET(menu);
 
     grid_menu_selected_id = 0;
     count = 0;
-    for (iter = 0; gridConfigs[iter].rows; iter++) {
+    for (iter = 0; iter < ARRAY_LEN(gridConfigs); iter++) {
         /* Don't show invalid configs */
         if (!gridConfigs[iter].valid) continue;
         tmp = g_strdup_printf("%d x %d grid",
                               gridConfigs[iter].rows,
                               gridConfigs[iter].columns);
 
-        menuitem = gtk_menu_item_new_with_label(tmp);
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
-        gtk_widget_show(menuitem);
+        ctk_drop_down_menu_append_item(menu, tmp, count);
 
         /* Update grid_config_id to set menu history */
         if (iter == grid_config_id) {
@@ -1471,9 +1507,7 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
         count++;
     }
 
-    gtk_option_menu_set_menu(GTK_OPTION_MENU(optionmenu), menu);
-    gtk_option_menu_set_history(GTK_OPTION_MENU(ctk_slimm->mnu_display_config),
-                                grid_menu_selected_id);
+    ctk_drop_down_menu_set_current_value(menu, grid_menu_selected_id);
 
     g_signal_connect(G_OBJECT(ctk_object->mnu_display_config), "changed",
                      G_CALLBACK(display_config_changed),
@@ -1481,7 +1515,7 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
 
     label = gtk_label_new("");
     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
-    gtk_box_pack_start(GTK_BOX(hbox), optionmenu, TRUE, TRUE, 5);
+    gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(menu), TRUE, TRUE, 5);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  
     table = gtk_table_new(20, 2, FALSE);
@@ -1512,8 +1546,8 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
 
     /* Option menu for resolutions */
     hbox = gtk_hbox_new(FALSE, 0);
-    optionmenu = gtk_option_menu_new();
-    ctk_slimm->mnu_display_resolution = optionmenu;
+    ctk_slimm->mnu_display_resolution =
+        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_COMBO);
 
     /* Create a drop down menu */
     setup_display_resolution_dropdown(ctk_object);
@@ -1529,9 +1563,9 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
 
 
     /* Option menu for refresh rates */
-    optionmenu = gtk_option_menu_new();
     hbox = gtk_hbox_new(FALSE, 0);
-    ctk_slimm->mnu_display_refresh = optionmenu;
+    ctk_slimm->mnu_display_refresh =
+        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_COMBO);
     setup_display_refresh_dropdown(ctk_object);
     g_signal_connect(G_OBJECT(ctk_object->mnu_display_refresh), "changed",
                      G_CALLBACK(display_refresh_changed),
@@ -1539,7 +1573,7 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
 
     label = gtk_label_new("");
     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
-    gtk_box_pack_end(GTK_BOX(hbox), optionmenu, TRUE, TRUE, 0);
+    gtk_box_pack_end(GTK_BOX(hbox), ctk_slimm->mnu_display_refresh, TRUE, TRUE, 0);
 
     gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, 3, 4, GTK_EXPAND | GTK_FILL,
                      GTK_EXPAND | GTK_FILL, 0.5, 0.5);
