@@ -136,6 +136,7 @@ GType ctk_thermal_get_type(void)
             sizeof (CtkThermal),
             0,    /* n_preallocs */
             NULL, /* instance_init */
+            NULL  /* value_table */
         };
 
         ctk_thermal_type =
@@ -428,7 +429,6 @@ static void cooler_control_state_toggled(GtkWidget *widget, gpointer user_data)
 {
     CtkThermal *ctk_thermal = CTK_THERMAL(user_data);
     gboolean enabled;
-    ReturnStatus ret;
     int value;
     gint result;
 
@@ -472,8 +472,8 @@ static void cooler_control_state_toggled(GtkWidget *widget, gpointer user_data)
 
     /* Update the server */
 
-    ret = NvCtrlSetAttribute(ctk_thermal->attribute_handle,
-                             NV_CTRL_GPU_COOLER_MANUAL_CONTROL, value);
+    NvCtrlSetAttribute(ctk_thermal->attribute_handle,
+                       NV_CTRL_GPU_COOLER_MANUAL_CONTROL, value);
 
     /* Update the GUI */
 
@@ -1661,7 +1661,7 @@ next_help:
     ctk_help_heading(b, &i, "ID");
     ctk_help_para(b, &i, __fan_id_help);
 
-    ctk_help_heading(b, &i, "Speed (%)");
+    ctk_help_heading(b, &i, "Speed (%%)");
     ctk_help_para(b, &i, __fan_speed_help);
 
     ctk_help_heading(b, &i, "Type");
