@@ -2,7 +2,7 @@
  * nvidia-settings: A tool for configuring the NVIDIA X driver on Unix
  * and Linux systems.
  *
- * Copyright (C) 2004 NVIDIA Corporation.
+ * Copyright (C) 2004,2012 NVIDIA Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU General Public License,
@@ -22,15 +22,15 @@
 
 #include "NVCtrlLib.h"
 
+#include "common-utils.h"
 #include "msg.h"
 
 #include "parse.h"
 
-#include <X11/extensions/xf86vmode.h>
-#include <X11/extensions/Xvlib.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
+#include <math.h> /* pow(3) */
 
 #include <sys/utsname.h>
 
@@ -46,11 +46,7 @@ NvCtrlAttributeHandle *NvCtrlAttributeInit(Display *dpy, int target_type,
 {
     NvCtrlAttributePrivateHandle *h = NULL;
 
-    h = calloc(1, sizeof (NvCtrlAttributePrivateHandle));
-    if (!h) {
-        nv_error_msg("Memory allocation failure!");
-        goto failed;
-    }
+    h = nvalloc(sizeof (NvCtrlAttributePrivateHandle));
 
     /* initialize the display and screen to the parameter values */
 
@@ -98,18 +94,16 @@ NvCtrlAttributeHandle *NvCtrlAttributeInit(Display *dpy, int target_type,
         if (subsystems & NV_CTRL_ATTRIBUTES_GLX_SUBSYSTEM) {
             h->glx = NvCtrlInitGlxAttributes(h);
         }
-        
-        /*
-         * initialize the XRandR extension and attributes; it is OK if
-         * this fails
-         */
-        
-        if (subsystems & NV_CTRL_ATTRIBUTES_XRANDR_SUBSYSTEM) {
-            h->xrandr = NvCtrlInitXrandrAttributes(h);
-        }
-
     } /* X Screen target type attribute subsystems */
-  
+
+    /*
+     * initialize the XRandR extension and attributes; this does not
+     * require an X screen and it is OK if this fails
+     */
+    if (subsystems & NV_CTRL_ATTRIBUTES_XRANDR_SUBSYSTEM) {
+        h->xrandr = NvCtrlInitXrandrAttributes(h);
+    }
+
     return (NvCtrlAttributeHandle *) h;
 
  failed:
@@ -519,7 +513,7 @@ ReturnStatus NvCtrlGetStringAttribute(NvCtrlAttributeHandle *handle,
 
 
 ReturnStatus NvCtrlSetStringAttribute(NvCtrlAttributeHandle *handle,
-                                      int attr, char *ptr, int *ret)
+                                      int attr, const char *ptr, int *ret)
 {
     if (!handle) return NvCtrlBadArgument;
     return NvCtrlSetStringDisplayAttribute(handle, 0, attr, ptr, ret);
@@ -531,9 +525,7 @@ ReturnStatus
 NvCtrlGetDisplayAttribute64(NvCtrlAttributeHandle *handle,
                             unsigned int display_mask, int attr, int64_t *val)
 {
-    ReturnStatus status;
     NvCtrlAttributePrivateHandle *h;
-    int value_32;
 
     h = (NvCtrlAttributePrivateHandle *) handle;
 
@@ -545,17 +537,22 @@ NvCtrlGetDisplayAttribute64(NvCtrlAttributeHandle *handle,
           case NV_CTRL_ATTR_EXT_VM_PRESENT:
             *val = (h->vm) ? True : False; break;
           case NV_CTRL_ATTR_EXT_XV_OVERLAY_PRESENT:
-            *val = (h->xv && h->xv->overlay) ? True : False; break;
+            *val = h->xv && h->xv->overlay; break;
           case NV_CTRL_ATTR_EXT_XV_TEXTURE_PRESENT:
-            *val = (h->xv && h->xv->texture) ? True : False; break;
+            *val = h->xv && h->xv->texture; break;
           case NV_CTRL_ATTR_EXT_XV_BLITTER_PRESENT:
-            *val = (h->xv && h->xv->blitter) ? True : False; break;
+            *val = h->xv && h->xv->blitter; break;
           default:
             return NvCtrlNoAttribute;
         }
         return NvCtrlSuccess;
     }
-    
+
+    if (attr >= NV_CTRL_ATTR_RANDR_BASE &&
+        attr <= NV_CTRL_ATTR_RANDR_LAST_ATTRIBUTE) {
+        return NvCtrlXrandrGetAttribute(h, display_mask, attr, val);
+    }
+
     if (((attr >= 0) && (attr <= NV_CTRL_LAST_ATTRIBUTE)) ||
         ((attr >= NV_CTRL_ATTR_NV_BASE) &&
          (attr <= NV_CTRL_ATTR_NV_LAST_ATTRIBUTE))) {
@@ -563,20 +560,6 @@ NvCtrlGetDisplayAttribute64(NvCtrlAttributeHandle *handle,
         return NvCtrlNvControlGetAttribute(h, display_mask, attr, val);
     }
 
-    if ((attr >= NV_CTRL_ATTR_XV_BASE) &&
-        (attr <= NV_CTRL_ATTR_XV_LAST_ATTRIBUTE)) {
-        status = NvCtrlXvGetAttribute(h, attr, &value_32);
-        *val = value_32;
-        return status;
-    }
-
-    if ((attr >= NV_CTRL_ATTR_XRANDR_BASE) &&
-        (attr <= NV_CTRL_ATTR_XRANDR_LAST_ATTRIBUTE)) {
-        status = NvCtrlXrandrGetAttribute(h, attr, &value_32);
-        *val = value_32;
-        return status;
-    }
-
     return NvCtrlNoAttribute;
     
 } /* NvCtrlGetDisplayAttribute64() */
@@ -603,44 +586,12 @@ NvCtrlSetDisplayAttribute(NvCtrlAttributeHandle *handle,
     NvCtrlAttributePrivateHandle *h;
 
     h = (NvCtrlAttributePrivateHandle *) handle;
-    
+
     if ((attr >= 0) && (attr <= NV_CTRL_LAST_ATTRIBUTE)) {
         if (!h->nv) return NvCtrlMissingExtension;
         return NvCtrlNvControlSetAttribute(h, display_mask, attr, val);
     }
 
-    if ((attr >= NV_CTRL_ATTR_XV_BASE) &&
-        (attr <= NV_CTRL_ATTR_XV_LAST_ATTRIBUTE)) {
-        
-        return NvCtrlXvSetAttribute(h, attr, val);
-    }
-    
-    if ((attr >= NV_CTRL_ATTR_XRANDR_BASE) &&
-        (attr <= NV_CTRL_ATTR_XRANDR_LAST_ATTRIBUTE)) {
-        
-        return NvCtrlXrandrSetAttribute(h, attr, val);
-    }
-
-    return NvCtrlNoAttribute;
-
-} /* NvCtrlSetDisplayAttribute() */
-
-
-ReturnStatus
-NvCtrlSetDisplayAttributeWithReply(NvCtrlAttributeHandle *handle,
-                                   unsigned int display_mask,
-                                   int attr, int val)
-{
-    NvCtrlAttributePrivateHandle *h;
-    
-    h = (NvCtrlAttributePrivateHandle *) handle;
-    
-    if ((attr >= 0) && (attr <= NV_CTRL_LAST_ATTRIBUTE)) {
-        if (!h->nv) return NvCtrlMissingExtension;
-        return NvCtrlNvControlSetAttributeWithReply(h, display_mask,
-                                                    attr, val);
-    }
-    
     return NvCtrlNoAttribute;
 }
 
@@ -679,12 +630,6 @@ NvCtrlGetValidDisplayAttributeValues(NvCtrlAttributeHandle *handle,
                                                       attr, val);
     }
 
-    if ((attr >= NV_CTRL_ATTR_XV_BASE) &&
-        (attr <= NV_CTRL_ATTR_XV_LAST_ATTRIBUTE)) {
-
-        return NvCtrlXvGetValidAttributeValues(h, attr, val);
-    }
-    
     return NvCtrlNoAttribute;
     
 } /* NvCtrlGetValidDisplayAttributeValues() */
@@ -827,7 +772,7 @@ NvCtrlGetStringDisplayAttribute(NvCtrlAttributeHandle *handle,
 ReturnStatus
 NvCtrlSetStringDisplayAttribute(NvCtrlAttributeHandle *handle,
                                 unsigned int display_mask,
-                                int attr, char *ptr, int *ret)
+                                int attr, const char *ptr, int *ret)
 {
     NvCtrlAttributePrivateHandle *h;
 
@@ -840,8 +785,7 @@ NvCtrlSetStringDisplayAttribute(NvCtrlAttributeHandle *handle,
     }
 
     return NvCtrlNoAttribute;
-
-} /* NvCtrlSetStringDisplayAttribute() */
+}
 
 
 ReturnStatus
@@ -861,12 +805,12 @@ NvCtrlGetBinaryAttribute(NvCtrlAttributeHandle *handle,
 ReturnStatus
 NvCtrlStringOperation(NvCtrlAttributeHandle *handle,
                       unsigned int display_mask, int attr,
-                      char *ptrIn, char **ptrOut)
+                      const char *ptrIn, char **ptrOut)
 {
     NvCtrlAttributePrivateHandle *h;
-    
+
     h = (NvCtrlAttributePrivateHandle *) handle;
-    
+
     if ((attr >= 0) && (attr <= NV_CTRL_STRING_OPERATION_LAST_ATTRIBUTE)) {
         if (!h->nv) return NvCtrlMissingExtension;
         return NvCtrlNvControlStringOperation(h, display_mask, attr, ptrIn,
@@ -874,8 +818,7 @@ NvCtrlStringOperation(NvCtrlAttributeHandle *handle,
     }
 
     return NvCtrlNoAttribute;
-
-} /* NvCtrlStringOperation() */
+}
 
 
 char *NvCtrlAttributesStrError(ReturnStatus status)
@@ -931,24 +874,6 @@ void NvCtrlAttributeClose(NvCtrlAttributeHandle *handle)
 } /* NvCtrlAttributeClose() */
 
 
-ReturnStatus
-NvCtrlXrandrSetScreenMode (NvCtrlAttributeHandle *handle,
-                           int width, int height, int refresh)
-{
-    return NvCtrlXrandrSetScreenMagicMode
-        ((NvCtrlAttributePrivateHandle *)handle, width, height, refresh);
-} /* NvCtrlXrandrSetScreenMode() */
-
-
-ReturnStatus
-NvCtrlXrandrGetScreenMode (NvCtrlAttributeHandle *handle,
-                           int *width, int *height, int *refresh)
-{
-    return NvCtrlXrandrGetScreenMagicMode
-        ((NvCtrlAttributePrivateHandle *)handle, width, height, refresh);
-} /* NvCtrlXrandrGetScreenMode() */
-
-
 /*
  * NvCtrlGetMultisampleModeName() - lookup a string desribing the
  * NV-CONTROL constant.
@@ -971,7 +896,9 @@ const char *NvCtrlGetMultisampleModeName(int multisample_mode)
         "8x (8xMS)",           /* FSAA_MODE_8xQ   */
         "16x (4xSS, 4xMS)",    /* FSAA_MODE_16xS  */
         "16x (8xMS, 8xCS)",    /* FSAA_MODE_16xQ  */
-        "32x (4xSS, 8xMS)"     /* FSAA_MODE_32xS  */
+        "32x (4xSS, 8xMS)",    /* FSAA_MODE_32xS  */
+        "32x (8xMS, 24xCS)",   /* FSAA_MODE_32x   */
+        "64x (16xSS, 4xMS)",   /* FSAA_MODE_64xS  */
     };
 
     if ((multisample_mode < NV_CTRL_FSAA_MODE_NONE) ||
@@ -982,3 +909,213 @@ const char *NvCtrlGetMultisampleModeName(int multisample_mode)
     return mode_names[multisample_mode];
 
 } /* NvCtrlGetMultisampleModeName  */
+
+
+ReturnStatus NvCtrlGetColorAttributes(NvCtrlAttributeHandle *handle,
+                                      float contrast[3],
+                                      float brightness[3],
+                                      float gamma[3])
+{
+    NvCtrlAttributePrivateHandle *h = (NvCtrlAttributePrivateHandle *) handle;
+
+    if (h->target_type == NV_CTRL_TARGET_TYPE_X_SCREEN) {
+        return NvCtrlVidModeGetColorAttributes(h, contrast, brightness, gamma);
+    } else if (h->target_type == NV_CTRL_TARGET_TYPE_DISPLAY) {
+        return NvCtrlXrandrGetColorAttributes(h, contrast, brightness, gamma);
+    }
+
+    return NvCtrlBadHandle;
+}
+
+ReturnStatus NvCtrlSetColorAttributes(NvCtrlAttributeHandle *handle,
+                                      float c[3],
+                                      float b[3],
+                                      float g[3],
+                                      unsigned int bitmask)
+{
+    ReturnStatus status;
+    int val = 0;
+
+    NvCtrlAttributePrivateHandle *h = (NvCtrlAttributePrivateHandle *) handle;
+
+    status = NvCtrlGetAttribute(h, NV_CTRL_ATTR_RANDR_GAMMA_AVAILABLE, &val);
+
+    if ((status != NvCtrlSuccess || !val) && 
+        h->target_type == NV_CTRL_TARGET_TYPE_X_SCREEN) {
+        return NvCtrlVidModeSetColorAttributes(h, c, b, g, bitmask);
+    } else if (status == NvCtrlSuccess && val &&
+               h->target_type == NV_CTRL_TARGET_TYPE_DISPLAY) {
+        return NvCtrlXrandrSetColorAttributes(h, c, b, g, bitmask);
+    }
+
+    return NvCtrlBadHandle;
+}
+
+
+ReturnStatus NvCtrlGetColorRamp(NvCtrlAttributeHandle *handle,
+                                unsigned int channel,
+                                uint16_t **lut,
+                                int *n)
+{
+    NvCtrlAttributePrivateHandle *h = (NvCtrlAttributePrivateHandle *) handle;
+
+    if (h->target_type == NV_CTRL_TARGET_TYPE_X_SCREEN) {
+        return NvCtrlVidModeGetColorRamp(h, channel, lut, n);
+    } else if (h->target_type == NV_CTRL_TARGET_TYPE_DISPLAY) {
+        return NvCtrlXrandrGetColorRamp(h, channel, lut, n);
+    }
+
+    return NvCtrlBadHandle;
+}
+
+
+/* helper functions private to the libXNVCtrlAttributes backend */
+
+void NvCtrlInitGammaInputStruct(NvCtrlGammaInput *pGammaInput)
+{
+    int ch;
+
+    for (ch = FIRST_COLOR_CHANNEL; ch <= LAST_COLOR_CHANNEL; ch++) {
+        pGammaInput->brightness[ch] = BRIGHTNESS_DEFAULT;
+        pGammaInput->contrast[ch]   = CONTRAST_DEFAULT;
+        pGammaInput->gamma[ch]      = GAMMA_DEFAULT;
+    }
+}
+
+/*
+ * Compute the gammaRamp entry given its index, and the contrast,
+ * brightness, and gamma.
+ */
+static unsigned short ComputeGammaRampVal(int gammaRampSize,
+                                          int i,
+                                          float contrast,
+                                          float brightness,
+                                          float gamma)
+{
+    double j, half, scale;
+    int shift, val, num;
+
+    num = gammaRampSize - 1;
+    shift = 16 - (ffs(gammaRampSize) - 1);
+
+    scale = (double) num / 3.0; /* how much brightness and contrast
+                                   affect the value */
+    j = (double) i;
+
+    /* contrast */
+
+    contrast *= scale;
+
+    if (contrast > 0.0) {
+        half = ((double) num / 2.0) - 1.0;
+        j -= half;
+        j *= half / (half - contrast);
+        j += half;
+    } else {
+        half = (double) num / 2.0;
+        j -= half;
+        j *= (half + contrast) / half;
+        j += half;
+    }
+
+    if (j < 0.0) j = 0.0;
+
+    /* gamma */
+
+    gamma = 1.0 / (double) gamma;
+
+    if (gamma == 1.0) {
+        val = (int) j;
+    } else {
+        val = (int) (pow (j / (double)num, gamma) * (double)num + 0.5);
+    }
+
+    /* brightness */
+
+    brightness *= scale;
+
+    val += (int)brightness;
+    if (val > num) val = num;
+    if (val < 0) val = 0;
+
+    val <<= shift;
+    return (unsigned short) val;
+}
+
+void NvCtrlUpdateGammaRamp(const NvCtrlGammaInput *pGammaInput,
+                           int gammaRampSize,
+                           unsigned short *gammaRamp[3],
+                           unsigned int bitmask)
+{
+    int i, ch;
+
+    /* update the requested channels within the gammaRamp */
+
+    for (ch = FIRST_COLOR_CHANNEL; ch <= LAST_COLOR_CHANNEL; ch++) {
+
+        /* only update requested channels */
+
+        if ((bitmask & (1 << ch)) == 0) {
+            continue;
+        }
+
+        for (i = 0; i < gammaRampSize; i++) {
+            gammaRamp[ch][i] =
+                ComputeGammaRampVal(gammaRampSize,
+                                    i,
+                                    pGammaInput->contrast[ch],
+                                    pGammaInput->brightness[ch],
+                                    pGammaInput->gamma[ch]);
+        }
+    }
+}
+
+void NvCtrlAssignGammaInput(NvCtrlGammaInput *pGammaInput,
+                            const float inContrast[3],
+                            const float inBrightness[3],
+                            const float inGamma[3],
+                            const unsigned int bitmask)
+{
+    int ch;
+
+    /* clamp input, but only the input specified in the bitmask */
+
+    for (ch = FIRST_COLOR_CHANNEL; ch <= LAST_COLOR_CHANNEL; ch++) {
+
+        /* only update requested channels */
+
+        if ((bitmask & (1 << ch)) == 0) {
+            continue;
+        }
+
+        if (bitmask & CONTRAST_VALUE) {
+            if (inContrast[ch] > CONTRAST_MAX) {
+                pGammaInput->contrast[ch] = CONTRAST_MAX;
+            } else if (inContrast[ch] < CONTRAST_MIN) {
+                pGammaInput->contrast[ch] = CONTRAST_MIN;
+            } else {
+                pGammaInput->contrast[ch] = inContrast[ch];
+            }
+        }
+
+        if (bitmask & BRIGHTNESS_VALUE) {
+            if (inBrightness[ch] > BRIGHTNESS_MAX) {
+                pGammaInput->brightness[ch] = BRIGHTNESS_MAX;
+            } else if (inBrightness[ch] < BRIGHTNESS_MIN) {
+                pGammaInput->brightness[ch] = BRIGHTNESS_MIN;
+            } else {
+                pGammaInput->brightness[ch] = inBrightness[ch];
+            }
+        }
+
+        if (bitmask & GAMMA_VALUE) {
+            if (inGamma[ch] > GAMMA_MAX) {
+                pGammaInput->gamma[ch] = GAMMA_MAX;
+            } else if (inGamma[ch] < GAMMA_MIN) {
+                pGammaInput->gamma[ch] = GAMMA_MIN;
+            } else {
+                pGammaInput->gamma[ch] = inGamma[ch];
+            }
+        }
+    }
+}
