@@ -4,22 +4,17 @@
  *
  * Copyright (C) 2004 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 #include "NvCtrlAttributes.h"
@@ -27,6 +22,7 @@
 
 #include "NVCtrlLib.h"
 
+#include "common-utils.h"
 #include "msg.h"
 
 #include "parse.h"
@@ -135,13 +131,9 @@ static Bool open_libgl(void)
 
     /* Initialize bookkeeping structure */
     if ( !__libGL ) {
-        __libGL = (__libGLInfo *) calloc(1, sizeof(__libGLInfo));
-        if ( !__libGL ) {
-            error_str = "Could not allocate memory.";
-            goto fail;
-        }
+        __libGL = nvalloc(sizeof(__libGLInfo));
     }
-    
+
 
     /* Library was already opened */
     if ( __libGL->handle ) {
@@ -370,10 +362,7 @@ get_fbconfig_attribs(NvCtrlAttributePrivateHandle *h)
     }
 
     /* Allocate to hold the fbconfig attributes */
-    fbcas = calloc(nfbconfigs + 1, sizeof(GLXFBConfigAttr));
-    if ( fbcas == NULL ) {
-        goto fail;        
-    }
+    fbcas = nvalloc((nfbconfigs + 1) * sizeof(GLXFBConfigAttr));
 
     /* Query each fbconfig's attributes and populate the attrib array */
     for ( i = 0; i < nfbconfigs; i++ ) {
