@@ -4,22 +4,17 @@
  *
  * Copyright (C) 2004 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 /*
@@ -43,6 +38,7 @@
  * Flag values used in the flags field of the AttributeTableEntry struct.
  */
 
+#define NV_PARSER_TYPE_HIJACK_DISPLAY_DEVICE   (1<<15)
 #define NV_PARSER_TYPE_FRAMELOCK               (1<<16)
 #define NV_PARSER_TYPE_COLOR_ATTRIBUTE         (1<<17)
 #define NV_PARSER_TYPE_NO_CONFIG_WRITE         (1<<18)
@@ -88,7 +84,7 @@
 #define NV_PARSER_STATUS_TARGET_SPEC_NO_TARGET_ID           13
 #define NV_PARSER_STATUS_TARGET_SPEC_BAD_TARGET_ID          14
 #define NV_PARSER_STATUS_TARGET_SPEC_TRAILING_GARBAGE       15
-
+#define NV_PARSER_STATUS_TARGET_SPEC_NO_TARGETS             16
 
 /*
  * define useful types
@@ -119,15 +115,34 @@ typedef struct _AttributeTableEntry {
 typedef struct _ParsedAttribute {
     char *display;
     char *name;
+    char *target_specification;
+    /*
+     * The target_type and target_id here are mostly set by the GUI to store
+     * target-specific information, as well as the cmd line for handling the
+     * case where an X screen is specified as part of the display (e.g.
+     * "localhost:0.1").  Note that if the target_specification is specified,
+     * the target_type and target_id are ignored when resolving to the list of
+     * targets that should be operated on.
+     */
     int target_type;
     int target_id;
+    char *target_name;
     int attr;
-    int val;
-    float fval; /* XXX put in a union with val? */
-    const float *pfval; /* XXX put in a union with val? */
+    union {
+        int i;
+        float f;
+        const float *pf;
+        char *str;
+    } val;
     uint32 display_device_mask;
     uint32 flags;
     struct _ParsedAttribute *next;
+    /*
+     * Upon being resolved, the ParsedAttribute's target_type and target_id,
+     * and/or target_specification get converted into a list of targets to
+     * which the attribute should be processed.
+     */
+    struct _CtrlHandleTargetNode *targets;
 } ParsedAttribute;
 
 
@@ -136,12 +151,12 @@ typedef struct _ParsedAttribute {
  * Attribute table; defined in parse.c
  */
 
-extern AttributeTableEntry attributeTable[];
+extern const AttributeTableEntry attributeTable[];
 
 
 /*
- * Indices into CtrlHandles->targets[] array; stored in
- * TargetTypeEntry.target_index.
+ * Indices into both targetTypeTable[] and CtrlHandles->targets[] array; stored
+ * in TargetTypeEntry.target_index.
  */
 
 #define X_SCREEN_TARGET  0
@@ -152,7 +167,8 @@ extern AttributeTableEntry attributeTable[];
 #define COOLER_TARGET    5
 #define THERMAL_SENSOR_TARGET 6
 #define NVIDIA_3D_VISION_PRO_TRANSCEIVER_TARGET 7
-#define MAX_TARGET_TYPES 8
+#define DISPLAY_TARGET   8
+#define MAX_TARGET_TYPES 9
 
 
 
@@ -190,7 +206,16 @@ typedef struct {
  * TargetType table; defined in parse.c
  */
 
-extern TargetTypeEntry targetTypeTable[];
+extern const TargetTypeEntry targetTypeTable[];
+extern const int targetTypeTableLen;
+
+/*
+ * accessor functions for getting target type info based on NV-CONTROL
+ * attribute type or by a name.
+ */
+const TargetTypeEntry *nv_get_target_type_entry_by_nvctrl(int nvctrl);
+const TargetTypeEntry *nv_get_target_type_entry_by_name(const char *name);
+
 
 /* nv_get_sdi_csc_matrxi() - Returns an array of floats that specifies
  * all the color, offset and scale values for specifing one of the
@@ -269,7 +294,7 @@ void nv_assign_default_display(ParsedAttribute *a, const char *display);
  * describing the error.
  */
 
-char *nv_parse_strerror(int);
+const char *nv_parse_strerror(int);
 
 int nv_strcasecmp(const char *, const char *);
 
@@ -324,6 +349,7 @@ char *nv_standardize_screen_name(const char *display_name, int screen);
 
 /* General parsing functions */
 
+int nv_parse_numerical(const char *start, const char *end, int *val);
 const char *parse_skip_whitespace(const char *str);
 void parse_chop_whitespace(char *str);
 const char *parse_skip_integer(const char *str);
@@ -332,6 +358,7 @@ const char *parse_read_integer_pair(const char *str,
                                     const char separator, int *a, int *b);
 const char *parse_read_name(const char *str, char **name, char term);
 const char *parse_read_display_name(const char *str, unsigned int *mask);
+const char *parse_read_display_id(const char *str, unsigned int *id);
 int parse_read_float_range(const char *str, float *min, float *max);
 int count_number_of_bits(unsigned int mask);
 
